/**
 * @file dialog.c Dialog functions
 * 
 * $Id: dialog.c,v 1.10 2001/12/31 09:26:54 chipx86 Exp $
 *
 * @Copyright (C) 2000-2002 The GNUpdate Project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include "genst.h"

static char *tempFilename = NULL;

int
dialog(char **result, const char *type, const char *title,
	   const char *text, const char *params)
{
	char  ans[4096];
	FILE *fp = NULL;
	char  cmdline[4096];
	int   ret = 0;

	if (tempFilename == NULL)
	{
		tempFilename = tmpnam(NULL);
	}
	
	/* Build the params */
	sprintf(cmdline, "dialog --backtitle \"" PACKAGE " v" VERSION
			"\" --nocancel");
	
	if (title != NULL)
		sprintf(cmdline, "%s --title \"%s\"", cmdline, title);

	sprintf(cmdline, "%s --%s \"\n%s\n\"", cmdline, type, text);

	if (params != NULL)
		sprintf(cmdline, "%s %s", cmdline, params);
	
	sprintf(cmdline, "%s 2>%s", cmdline, tempFilename);
	
	strcpy(ans, "");

	for (;;)
	{
		/* Redirect stderr */
		
		ret = system(cmdline) / 255;

		if (ret < 255)
		{
			fp = fopen(tempFilename, "r");
			
			if (fp == NULL)
				continue;
			
			while(!feof(fp))
			{
				char buffer[2];
				char c = fgetc(fp);

				if (c != -1)
				{
					buffer[0] = c;
					buffer[1] = '\0';

					strcat(ans, buffer);
				}
			}

			fclose(fp);

			unlink(tempFilename);
			
			if (result != NULL)
				*result = strdup(ans);
			
			return (ret == 0);
		}
	}

	return -1; /* This should never be reached. */
}

void
dialogMsgBox(const char *text)
{
	dialog(NULL, "msgbox", NULL, text, "0 0");
}

char
dialogYesNo(const char *title, const char *text)
{
	return dialog(NULL, "yesno", title, text, "0 0");
}

char *
dialogInputBox(const char *title, const char *text, const char *defaultValue)
{
	int ret;
	char *buffer = NULL;
	char params[4096];

	sprintf(params, "0 0");

	if (defaultValue != NULL)
		sprintf(params, "%s %s", params, defaultValue);
	
	for (;;)
	{
		ret = dialog(&buffer, "inputbox", title, text, params);

		if (strlen(buffer) > 0)
			return buffer;

		free(buffer);

		if (confirmQuit())
			quit();
	}

	return NULL; /* Never reached. */
}


/**************************************************************************
 * DialogList
 **************************************************************************/
DialogList *
newDialogList(const char *title, const char *text)
{
	DialogList *list;

	list = (DialogList *)malloc(sizeof(DialogList));

	list->title    = strdup(title);
	list->text     = strdup(text);
	list->count    = 0;
	list->items    = NULL;
	list->lastItem = NULL;

	return list;
}

void
freeDialogList(DialogList *list)
{
	ListItem *curItem  = NULL;
	ListItem *nextItem = NULL;

	if (list == NULL)
		return;

	for (curItem = list->items; curItem != NULL; curItem = nextItem)
	{
		nextItem = curItem->next;
		freeDialogListitem(curItem);
	}

	list->items    = NULL;
	list->lastItem = NULL;
	
	if (list->title != NULL) free(list->title);
	if (list->text  != NULL) free(list->text);

	free(list);
}

void
dialogListAddItem(DialogList *list, ListItem *item)
{
	if (list == NULL || item == NULL)
		return;

	if (list->items == NULL)
		list->items = item;

	item->prev = list->lastItem;
	
	if (list->lastItem != NULL)
		list->lastItem->next = item;

	list->lastItem = item;
	
	list->count++;
}

int
dialogRenderCheckList(DialogList *list, char ***result)
{
	char  params[4096];
	char *res;
	int   count;
	ListItem *item;

	sprintf(params, "15 60 5");
	
	for (item = list->items; item != NULL; item = item->next)
	{
		sprintf(params, "%s \"%s\" \"%s\" %s", params, item->name,
				(item->description == NULL ? "" : item->description),
				(item->on ? "ON" : "off"));
	}

	dialog(&res, "checklist", list->title, list->text, params);

	count = charsToArray(res, result);
	
	free(res);

	return count;
}

char *
dialogRenderRadioList(DialogList *list)
{
	char params[4096];
	char *result;
	ListItem *item;

	sprintf(params, "15 60 5");
	
	for (item = list->items; item != NULL; item = item->next)
	{
		sprintf(params, "%s \"%s\" \"%s\" %s", params, item->name,
				(item->description == NULL ? "" : item->description),
				(item->on ? "ON" : "off"));
	}

	dialog(&result, "radiolist", list->title, list->text, params);

	return result;
}


/**************************************************************************
 * List Items
 **************************************************************************/
ListItem *
newDialogListItem(const char *name, const char *description, char on)
{
	ListItem *item;

	item = (ListItem *)malloc(sizeof(ListItem));

	if (name != NULL)
		item->name = strdup(name);
	else
		item->name = NULL;

	if (description != NULL)
		item->description = strdup(description);
	else
		item->description = NULL;
	
	item->on          = on;
	item->prev        = NULL;
	item->next        = NULL;
	
	return item;
}

void
freeDialogListitem(ListItem *item)
{
	if (item == NULL)
		return;
	
	if (item->prev != NULL) item->prev->next = item->next;
	if (item->next != NULL) item->next->prev = item->prev;
	
	if (item->name        != NULL) free(item->name);
	if (item->description != NULL) free(item->description);

	free(item);
}

/**************************************************************************
 * Menu
 **************************************************************************/
DialogMenu *
newDialogMenu(const char *title, const char *text)
{
	DialogMenu *menu;

	menu = (DialogMenu *)malloc(sizeof(DialogMenu));

	menu->defaultSelection = NULL;
	menu->title    = strdup(title);
	menu->text     = strdup(text);
	menu->count    = 0;
	menu->items    = NULL;
	menu->lastItem = NULL;

	return menu;
}

void
freeDialogMenu(DialogMenu *menu)
{
	MenuItem *curItem  = NULL;
	MenuItem *nextItem = NULL;

	if (menu == NULL)
		return;

	for (curItem = menu->items; curItem != NULL; curItem = nextItem)
	{
		nextItem = curItem->next;
		freeDialogMenuItem(curItem);
	}

	menu->items    = NULL;
	menu->lastItem = NULL;
	
	if (menu->defaultSelection != NULL) free(menu->defaultSelection);
	if (menu->title            != NULL) free(menu->title);
	if (menu->text             != NULL) free(menu->text);

	free(menu);
}

void
dialogAddMenuItem(DialogMenu *menu, MenuItem *item)
{
	if (menu == NULL || item == NULL)
		return;

	if (menu->items == NULL)
		menu->items = item;

	item->prev = menu->lastItem;
	
	if (menu->lastItem != NULL)
		menu->lastItem->next = item;

	menu->lastItem = item;
	
	menu->count++;
}

void
dialogAddMenuSep(DialogMenu *menu)
{
	MenuItem *item;

	item = newDialogMenuItem("--", "");
	dialogAddMenuItem(menu, item);
}

void
dialogSetMenuDefault(DialogMenu *menu, const char *defaultSelection)
{
	if (menu == NULL)
		return;

	if (menu->defaultSelection != NULL)
		free(menu->defaultSelection);
	
	if (defaultSelection == NULL)
		menu->defaultSelection = NULL;
	else
		menu->defaultSelection = strdup(defaultSelection);
}

char *
dialogRenderMenu(DialogMenu *menu)
{
	MenuItem *item;
	char params[4096];
	char *res;
	char *dialogname;
	int ret;

	/* NASTY hack */
	if (menu->defaultSelection != NULL)
	{
		dialogname = (char *)malloc((strlen(menu->defaultSelection) +
									 23) * sizeof(char));

		sprintf(dialogname, "default-item \"%s\" --menu",
				menu->defaultSelection);
	}
	else
	{
		dialogname = (char *)malloc(5 * sizeof(char));
		strcpy(dialogname, "menu");
	}
	
	sprintf(params, "18 55 8");
	
	for (item = menu->items; item != NULL; item = item->next)
	{
		sprintf(params, "%s \"%s\" \"%s\"", params, item->name,
				(item->description == NULL ? "" : item->description));
	}

	for (;;)
	{
		ret = dialog(&res, dialogname, menu->title, menu->text, params);

		if (ret == 0)
		{
			free(res);

			if (confirmQuit())
				quit();
		}
		else if (!isMenuSep(res))
		{
			free(dialogname);

			return res;
		}

		free(res);
	}
}

MenuItem *
newDialogMenuItem(const char *name, const char *description)
{
	MenuItem *item;

	item = (MenuItem *)malloc(sizeof(MenuItem));

	if (name != NULL)
		item->name = strdup(name);
	else
		item->name = NULL;

	if (description != NULL)
		item->description = strdup(description);
	else
		item->description = NULL;

	item->prev = NULL;
	item->next = NULL;
	
	return item;
}

void
freeDialogMenuItem(MenuItem *item)
{
	if (item == NULL)
		return;
	
	if (item->prev != NULL) item->prev->next = item->next;
	if (item->next != NULL) item->next->prev = item->prev;
	
	if (item->description != NULL) free(item->description);
	if (item->name        != NULL) free(item->name);

	free(item);
}

Gauge *
newDialogGauge(const char *title, const char *text)
{
	Gauge *gauge;
	char *cmdline;

	gauge = (Gauge *)malloc(sizeof(Gauge));

	cmdline = (char *)malloc((strlen(title) + strlen(text) + 39) *
							 sizeof(char));

	strcpy(cmdline, "dialog --title \"");
	strcat(cmdline, title);
	strcat(cmdline, "\" --gauge \"\n");
	strcat(cmdline, text);
	strcat(cmdline, "\n\" 20 70 0");
	
	gauge->stream = popen(cmdline, "w");

	free(cmdline);
	
	if (gauge->stream == NULL)
	{
		fprintf(stderr, "Unable to open stream to gauge.");
		exit(1);
	}

	/* Turn off buffering. */
	setvbuf(gauge->stream, NULL, _IONBF, 0);
	
	return gauge;
}

void
freeDialogGauge(Gauge *gauge)
{
	if (gauge == NULL)
		return;

	dialogStopGauge(gauge);
	
	free(gauge);
}

void
dialogStopGauge(Gauge *gauge)
{
	pclose(gauge->stream);

	gauge->stream = NULL;
}

void
dialogUpdateGauge(Gauge *gauge, int pct, char *text)
{
	if (gauge == NULL || text == NULL)
		return;
	
	fprintf(gauge->stream, "XXX\n");
	fprintf(gauge->stream, "%d\n", pct);
	fprintf(gauge->stream, "\n%s\n", text);
	fprintf(gauge->stream, "XXX\n");
}

